/**
 *            Pizza:                            [Abstrata]
 *                 imprimeIngredientes()        [virtual puro]
 *                 preco()                      [virtual puro]
 *                 tempoPreparo()               [virtual puro]
 *                 tamanho()                    [virtual puro]
 *
 *            PizzaCalabresa: Pizza
 *
 *            PizzaMussarela: Pizza
 *
 *            Pizza4Queijos:  Pizza
 *
 *            Forno
 *               Pizza
 *               prepararPizza()         // acessar tempoPreparo()
 */
#include <iostream>
#include <string>
using namespace std;
// Classe Abstrata
class Pizza {
public:
     virtual void imprimeIngredientes() = 0;       
     virtual float preco() = 0;                      
     virtual int tempoPreparo() = 0;  
     virtual int tamanho() = 0;    
};

class PizzaCalabresa : public Pizza {
     virtual void imprimeIngredientes() {
        cout << "Massa, calabresa, mussarela, cebola, tomate, oregano, molho de tomate." << endl;
     }       
     virtual float preco() {
        return 35;
     }                     
     virtual int tempoPreparo() {
        return 30; // minutos
     }  
     virtual int tamanho() {
        return 40; // cm de diâmetro
     }
};

class PizzaMussarela : public Pizza {
public:
     virtual void imprimeIngredientes() {
        cout << "Massa, mussarela, oregano, tomate e molho de tomate." << endl;
     }       
     virtual float preco() {
        return 20; 
     }                      
     virtual int tempoPreparo() {
        return 25; // minutos
     }  
     virtual int tamanho() {
        return 40;
     }    
};

class Pizza4Queijos : public Pizza {
public:
     virtual void imprimeIngredientes() {
        cout << "Massa, mussarela, parmesão, gorgonzola, catupiry, oregano e tomate." << endl;
     }      
     virtual float preco() {
        return 49.90; // reais
     }                      
     virtual int tempoPreparo() {
        return 20; // minutos
     }  
     virtual int tamanho() {
        return 40; // cm
     }    
};

class Forno {
private:
    Pizza * pizza;
public:
    Forno() {}    
    Forno( Pizza * pizza ) : pizza( pizza ) {}
    void setPizza( Pizza * pizza ) {
        this->pizza = pizza;
    }
    void prepararPizza() {
        pizza->imprimeIngredientes();
        cout << "Tempo de preparo: " << pizza->tempoPreparo() << " minutos" << endl;
        cout << "Tamanho: " << pizza->tamanho() << " cm" << endl;
        cout << "Valor: " << pizza->preco() << " reais" << endl;
    }    
};

int main() {
    int opcao = 0;
    cout << "Mussarela [1] Calabresa [2] 4 Queijos [3]: ";
    cin  >> opcao;
    Forno f;
    if( opcao == 1 )
        f.setPizza( new PizzaMussarela() );
    else if( opcao == 2 )
        f.setPizza( new PizzaCalabresa() );
    else if( opcao == 3 )
        f.setPizza( new Pizza4Queijos() );
    
    f.prepararPizza();
}
